# %Z% %M% %I%.%B% %E% %U%
# %Z% %Q%

###########################################################################
#
# Licensed Materials - Property of IBM
# 5724-T05
#
# (C) Copyright IBM Corp. 2003,2008. All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
###########################################################################

## Lucent UMTS CS XML data hash module.
## Define all the Global Variables which will be used by all the
## subroutines.

use strict;

my $idxHash = {};
my $cntHash = {};
my ($keyName,$keyVal);
my $gDataFile;
my $gMeasDate;


sub build_data_hash {

  my ($ppConfig, $output, $insep, $config) = @_;

  my ($file_content, $meas_time, $cnt_name, $cnt_value, $perlen, $moid, $moid_entity );
  my ($CallServer_name, $CallServer_id);
  my $local_node_id;

  my %missing_counters;
  # Keeps track of counters in the input that are neither in ppConfig.txt
  # nor in counts_to_ignore.txt. We print an error message about each such counter,
  # but only the first time we see it.

  my $create_jobidx_cache= {}; 
  # We have to create job idx to fit with the Prospect hierarchy,
  # $rewrite_jobidx_cache caches the results (!) so that we don't have to
  # call the create subroutine more than once for a given MOID.

  my $create_jobidx_or_not= {};
  # $create_jobidx_or_not keeps track of whether we succeeded in creating
  # a given MOID or not. 
  #
  # The value will be one of -
  # undef - this job idx has not been created.
  # "yes"      - this job idx has been created successfuly
  # "failed"   - this job idx has failed to be created

  my %entity_remapping= 
    (
     # Some general rules for loading counters on different entities.
     # Sometimes we want to rename an entity, or load all its counters
     # on another entity.
     # Sometimes which entity we load the counters on depends on
     # which entity appears above it in the MOID.
     # Not all entities fit this pattern, but this
     # cuts down on the number of chained if ... elsif
     # statements.
     IN =>           {always =>     "IN_Service"},
     # The above rule says if the entity "IN" appears in the input,
     # pretend the input says "IN_Service" instead.
     # That is, load all counters for "IN" on "IN_Service".
     CDPA =>         {always =>     "SigPoint_DestGT"},
     DPC =>          {sometimes => {"CIC" => "SigPoint_DestPC_CIC",
				    "Stack" => "SCCP_DPC"},
		      otherwise =>  "SigPoint_DestPC"},
     # The above rule says if the entity "DPC" appears in the input,
     # check what its parent is (which entity appears before DPC in
     # the MOID). This is the meaning of the {sometimes} element.
     # If the parent is "CIC", load the DPC counters on
     # "SigPoint_DestPC_CIC". If the parent is "Stack", load the
     # counters on "SCCP_DPC".
     # The {otherwise} element means that if the parent is not
     # "CIC" or "Stack", load the counters on "SigPoint_DestPC".
     # Note that if {sometimes} is present, {otherwise} is optional.
     # If {otherwise} is absent, the entity doesn't change if the
     # parent isn't one of the options under {sometimes}.
     PointCode =>    {always =>     "SigPoint_DestPC"},
     SCCP =>         {always =>     "SigPoint_SCCP"},
     SSN =>          {sometimes => {DPC => "SigPoint_DestPC_SSN",
				    SCCP => "SigPoint_SCCP_SSN",
				    ServiceMember => "SigPoint_DestPC",
				    TCAP => "TCAP_SSN"}
		     },
     # Note that the above rule has a {sometimes} element without
     # an {otherwise}. If the parent entity of "SSN" isn't one of "DPC",
     # "SCCP", "ServiceMember" or "TCAP", we load the counters on "SSN".
     TCAP =>         {always =>     "SigPoint_TCAP"},
     M3UASG =>       {always =>     "SigTran_SG"},
     SCTPAssoc =>    {always =>     "SigTran_SCTPAssoc"},
     oid =>          {sometimes => {CesHost => "SNMP_Interface",
				    CesServiceMember => "SCTP_Stack"}
		     },
     Host =>         {always =>     "ServiceMember"},
     ChannelGroup => {sometimes => {Mgw => "Mgw_ChannelGroup"}},
     BearerType =>   {sometimes => {CGName => "CG_MGW_BearerType"}},
     serviceType =>  {always =>     "ServiceMember"},
     LAC =>          {sometimes => {PLMN => "LAC"},
		      otherwise => "LAC_Paging"},
     BearerCallType => {sometimes => {BSC => "BSC_BearerType_CS",
				      LAC => "LAC_PG_BearerType",
				      RNC => "RNC_BearerType_CS"}
		       }
    );
			

  my $moid_separator= $ppConfig->{CONFIG}{MOIDSEPARATOR};

  my ($seen_timestamp, $filetype, $moid_prefix);

  my $skip_ahead = "yes";
  my $CGName = "";
  my $MediaGW_CS_Label ="";
  my $servicemember_label="";
  my $linkset_label="";
  my ($t_plmn, $t_lac, $t_cell_rnc);
  my ($plmn, $lac, $rnc);

  while ($file_content = <STDIN>) {
    chomp($file_content);

    my ($line_type, $rest)= $file_content =~ /^([^$insep]*)$insep(.*)$/o;
    if ($line_type eq "FILENAME") {
      $skip_ahead= "no";

      my $file_name = $rest;

      ($CallServer_name, $CallServer_id, $meas_time) = parse_filename($file_name, $config, $ppConfig);
      $file_name =~ s/"/\\"/;
      $CallServer_name =~ s/"/\\"/;
      $ppConfig->{CallServer_NAME} = $CallServer_name;

      if (not defined $CallServer_id or not defined $meas_time) {
	# No point parsing this file if we cannot extract the CallServer Id or the MEAS_TIME
	messageHandler($config, 1, "Skipping ...");
	$skip_ahead="yes";
      } # endif not defined meas_time

    } elsif ($line_type eq "LABEL") {
      my $CallServer_label = $rest;

      $CallServer_label =~ tr/-!/~|/;
      $CallServer_label =~ s/"/\\"/;

      $ppConfig->{CallServer_LABEL} = $CallServer_label;
    } else {
      # $line_type is not "FILENAME"
      next if $skip_ahead eq "yes";

      if ($line_type eq "MOID") {

	($moid, $perlen)= split /$insep/o, $rest;
	$moid =~ s/, +/,/g;
	if ($moid eq "") {
	  $moid_entity = "";
        } else {
	  my $previous_entity;
	  ($moid_entity) = $moid =~ /(\w+)=[^,]+$/; 

	  #-----------------------------------
	  # Populate value in serviceType into ServiceMemberLabel
	  #-----------------------------------
	  ##get servicemember_label from ServiceMember attribute
          if ($moid =~ /^Service=(.+),ServiceMember=(.*?)$/) {
	    ($servicemember_label) = $1; 

	    ##get servicemember_label from serviceType attribute have FileSystem
	  } elsif ($moid =~ /^Cabinet=(.*),serviceType=(.+),FileSystem/) {
	    ($servicemember_label) = $2; 

	    ##get servicemember_label from serviceType attribute 
	  } elsif ($moid =~ /^Cabinet=(.*),serviceType=(.+)$/) {
	    ($servicemember_label) = $2;
	  } 
	  if (defined($servicemember_label)){
	    $servicemember_label =~ s/"/\\"/;
	  }

	  #-----------------------------------
	  # Populate value in LSName into LinkSetLabel
	  #-----------------------------------
	  if ($moid =~ /^Service=(.*),LSName=(.+),HSL/) {
	    ($linkset_label) = $2; 
	  } elsif ($moid =~ /^Service=(.*),LSName=(.+),Link/) {
	    ($linkset_label) = $2; 
	  } 
	  elsif ($moid =~ /^Service=(.*),LSName=(.+)$/) {
	    ($linkset_label) = $2; 
	  }
	  if (defined($linkset_label)){
 	    $linkset_label =~ s/"/\\"/;
	  }

	  #-----------------------------------
	  # Populate value in mgwName into MediaGW_CS_Label
	  #-----------------------------------
	  if ($moid_entity eq "mgwName"){
	    ($MediaGW_CS_Label) = $moid =~ /\w+,mgwName=(.+)/;
	  }
	  if(defined($MediaGW_CS_Label)){
	     $MediaGW_CS_Label =~ s/"/\\"/;
	  }
	
	  #--------------------------------------
	  # add for LucentUMTS_RP8P20
	  # Get PLMN=<>, LAC=<>, RNC=<>, BearerCallType=<>
	  # for moid_entity = RNC_BearerType
	  if ($moid =~ /\w+,PLMN=(.+),LAC=(.+),RNC=(.+),BearerCallType=(.+)$/) {
	    ($plmn, $lac, $rnc) = $moid =~ /\w+,PLMN=(.+),LAC=(.+),RNC=(.+),[^=]+=(.+)$/;
	  }

	  #--------------------------------------
	  # Get T_PLMN, T_LACID and T_CallID/T_RNCID 
	  # for moid_entity = T_CELL_RNC or T_C_R_Cause
	  #--------------------------------------
	  # measObjLdn have as below
	  # T_PLMN=<>,T_LACID=<>,T_CellID|T_RNCID=<>,U_Cause|Map_Cause|G_Cause=<>
	  #--------------------------------------
	  #SEAde68412#
	  #need to support T_PLMN=<>,T_LACID|T_LAC=<>,T_CellID|T_RNCID|T_CELL|T_Cell|T_RNC=<>,U_Cause|Map_Cause|G_Cause=<>
	  #change the pattern matching to T_PLMN=<>,[t_lac]=<>,[t_cell|t_rnc]=<>,[u|map|g_cause]=<>
	  if ($moid =~ /\w+,T_PLMN=(.+),[^=]+=(.+),[^=]+=(.+),[^=]+=(.+)$/){
	    ($t_plmn, $t_lac, $t_cell_rnc) = ($1, $2, $3, $4); 
	  }
	  #--------------------------------------
	  # measObjLdn have as below
	  # T_PLMN=<>,T_LACID=<>,T_CellID|T_RNCID=<>
	  #--------------------------------------
	  #SEAde68412#
	  #need to support T_PLMN=<>,T_LACID|T_LAC=<>,T_CellID|T_RNCID|T_CELL|T_Cell|T_RNC=<>
	  #change the pattern matching to T_PLMN=<>,[t_lac]=<>,[t_cell|t_rnc]=<>,[u|map|g_cause]=<>
	  elsif ($moid =~ /\w+,T_PLMN=(.+),[^=]+=(.+),[^=]+=(.+)$/){
	    ($t_plmn, $t_lac, $t_cell_rnc) = ($1, $2, $3); 
	  }

	  #-----------------------------------
	  # Populate value in CGName into Label
	  #-----------------------------------
	  if ($moid =~ /\w+,CGName=(.+),BearerType/){
	    ($CGName) = ($1);
	  }
	  if ($moid_entity eq "CGName") {
	    ($moid_entity, $CGName) = $moid =~ /(\w+)=[^,]+,CGName=([^,]+)$/;
	    ($moid) = $moid =~ /(.*),CGName=[^,]+$/;
	  }
	  if(defined($CGName)){
	    $CGName =~ s/"/\\"/;
	  }
	  #messageHandler($config, 3, "moid_entity ", $moid_entity);

	  if ($moid_entity eq "TechnologyType") {
	    ($moid_entity) = $moid =~ /(\w+)=[^,]+,TechnologyType=[^,]+$/;
	  } 
	  elsif ($moid_entity eq "Target" or $moid_entity eq "Mgw") {
	    # If the moid entity is not unique on its own:
	    #   Add previous entity to moid
	    my ($previous_entity) = $moid =~ /(\w+)=[^,]+,$moid_entity=[^,]+$/;
	    $moid_entity .= "_${previous_entity}";
	  } 
	  # Now some general rules for changing $moid_entity.
	  elsif (exists $entity_remapping{$moid_entity}) {
	    my $ref= \%{$entity_remapping{$moid_entity}};
	    if (exists $ref->{always}) {
	      $moid_entity= $ref->{always};
	    } else {
	      my ($parent_entity)= $moid =~ /(\w+)=[^,]+,\w+=[^,]*$/;
	      # Set $parent_entity to the last-but-one class name.
	      if (exists $ref->{sometimes}->{$parent_entity}) {
		$moid_entity= $ref->{sometimes}->{$parent_entity};
	      } elsif (exists $ref->{otherwise}) {
		$moid_entity= $ref->{otherwise};
	      }
	      if ($moid_entity eq "BSC_BearerType_CS") {
		# BearerCallType sometimes looks like:
		# ServiceMember=w,LAC=x,BSC=y,BearerType=z
		# (which we want to load on LAC_BearerType_CS)
		# and sometimes:
		# ServiceMember=w,BSC=y,BearerType=z
		# (which we want to load on BSC_BearerType_CS).
		# The rules allowed by %entity_remapping aren't flexible enough.
		# They just see the parent of BearerType as BSC in both cases.
		# So assume we're going to load on BSC_BearerType_CS and
		# then check whether the parent of BSC (grandparent of BearerType)
		# is LAC. If it is, change the entity to LAC_BSC_BearerType.
		my ($grandparent_entity)= $moid =~ /(\w+)=[^,]+,\w+=[^,]*,\w+=[^,]*$/;
		if ($grandparent_entity eq "LAC") {
		  $moid_entity= "LAC_BSC_BearerType";
		}
	      } elsif ($moid_entity eq "RNC_BearerType_CS") {
		# See above.
		# Sometimes BearerCallType looks like:
		# ServiceMember=w,LAC=x,RNC=y,BearerType=z
		# (which we want to load on LAC_RNC_BearerType)
		# and sometimes:
		# ServiceMember=w,RNC=y,BearerType=z
		# (which we want to load on RNC_BearerType_CS).
		my ($grandparent_entity)= $moid =~ /(\w+)=[^,]+,\w+=[^,]*,\w+=[^,]*$/;
		if ($grandparent_entity eq "LAC") {
		  $moid_entity= "LAC_RNC_BearerType";
		}		
		# but sometimes:
		# ServiceMember=w,PLMN=v,LAC=x,RNC=y,BearerType=z
		# (which we want to load on RNC_BearerType)
		my ($greatgrandparent_entity)= $moid  =~ /(\w+)=[^,]+,\w+=[^,]*,\w+=[^,]*,\w+=[^,]*$/;
		if ($greatgrandparent_entity eq "PLMN") {
		  $moid_entity= "RNC_BearerType";
		}		
	      }
	    } # if exists ... {always}
	  } # elsif exists $entity_remapping

        }
	
      } else {

	# If the first field on the line is not "FILENAME" or "MOID", 
	# it must be a counter name.

	($cnt_name, $cnt_value)= ($line_type, $rest);
        # Remove the VS. prefix
        $cnt_name =~ s/^VS\.//;

        #SEAde68523 -make case not sensitives
	$cnt_name = uc $cnt_name;

        # If moid contains technology type:
        #   Add the technology type to counter name
	#
        my ($technology_type) = $moid =~ /TechnologyType=([^,]+)/;

	if (defined $technology_type) { 
	  #
	  # add for LucentUMTS_RP8P20
	  $technology_type = uc($technology_type); 
	  $cnt_name .= "_${technology_type}";
        }

        # If moid contains service=ims && counter name is channelAnsUsed:
        #   Add _IMS to the counter name
	#
        my ($service_name) = $moid =~ /Service=([^,]+)/;
	if (defined $service_name && (uc($service_name) eq "IMS" && $cnt_name eq "channelAnsUsed")) {
	  $cnt_name .= "_IMS";
	}

        my $jobidx;

	# set $measgroup to the last class name in the MOID.
        if (exists $ppConfig->{COUNTER_TO_GROUP}{$moid_entity} &&
            exists $ppConfig->{COUNTER_TO_GROUP}{$moid_entity}{$cnt_name} ) {
	  my $meas_group = $ppConfig->{COUNTER_TO_GROUP}{$moid_entity}{$cnt_name};

	  if (not exists $create_jobidx_or_not->{$meas_group} or
              not exists $create_jobidx_or_not->{$meas_group}{$moid}) {

	    # Assume that we can't create the job idx until we've succeeded
	    # in rewriting it.
	    $create_jobidx_or_not->{$meas_group}{$moid} = "failed";

            my $module = $ppConfig->{MODLIST}{$meas_group};
	    #messageHandler($config, 3, "modList for $meas_group is ", $module);

            if ($module eq "none") {
              $jobidx = $CallServer_id;
            } else {
	      $jobidx= &{$config->{REWRITE_SUB_REFS}->{$module}}
	      ($config, $CallServer_id, $moid);
	    }

            if (defined $jobidx) {

              $create_jobidx_or_not->{$meas_group}{$moid} = "done";
              # Place it in the cache so we can utilise it next time
              $create_jobidx_cache->{$meas_group}{$moid} = $jobidx;

	      #Add the CGName to ChannelGroup
	      # It is done here so we only do it once
	      #SEAde68523 need to change the label to upper case
	      if($meas_group eq "ChannelGroup1" || $meas_group eq "ChannelGroup_MGW1" || $meas_group eq "CG_MGW_BearerType1" )
		{
		  my $field_pos=$ppConfig->{COUNTGROUPFIELD}{$meas_group}{'LABEL'};
		  $output->{$meas_time}{$perlen}{$meas_group}{$jobidx}{$field_pos} = $CGName;
		}
	      if(exists ($ppConfig->{COUNTGROUPFIELD}{$meas_group}{'MEDIAGW_CS_LABEL'}) && defined($MediaGW_CS_Label))
		{
		  my $field_pos=$ppConfig->{COUNTGROUPFIELD}{$meas_group}{'MEDIAGW_CS_LABEL'};
		  $output->{$meas_time}{$perlen}{$meas_group}{$jobidx}{$field_pos} = $MediaGW_CS_Label;
		}
	      ##add service member label to all measObjLdn contain ServiceMember
	      if (exists ($ppConfig->{COUNTGROUPFIELD}{$meas_group}{'SERVICEMEMBERLABEL'}) &&  defined($servicemember_label)){
		my $field_pos=$ppConfig->{COUNTGROUPFIELD}{$meas_group}{'SERVICEMEMBERLABEL'};
		$output->{$meas_time}{$perlen}{$meas_group}{$jobidx}{$field_pos} = $servicemember_label;
	      }
	      ##add linkset label to all measObjLdn contain ServiceMember
	      if (exists ($ppConfig->{COUNTGROUPFIELD}{$meas_group}{'LINKSET_LABEL'}) &&  defined($linkset_label)) {
		my $field_pos=$ppConfig->{COUNTGROUPFIELD}{$meas_group}{'LINKSET_LABEL'};
		$output->{$meas_time}{$perlen}{$meas_group}{$jobidx}{$field_pos} = $linkset_label;
	      }
	      ##add t_plmn, t_lac and t_cell_rnc
	      if (exists ($ppConfig->{COUNTGROUPFIELD}{$meas_group}{'T_PLMN'}) &&  defined($t_plmn)) {
		my $field_pos=$ppConfig->{COUNTGROUPFIELD}{$meas_group}{'T_PLMN'};
		$output->{$meas_time}{$perlen}{$meas_group}{$jobidx}{$field_pos} = $t_plmn;
	      }
	      if (exists ($ppConfig->{COUNTGROUPFIELD}{$meas_group}{'T_LAC'}) &&  defined($t_lac)){
		my $field_pos=$ppConfig->{COUNTGROUPFIELD}{$meas_group}{'T_LAC'};
		$output->{$meas_time}{$perlen}{$meas_group}{$jobidx}{$field_pos} = $t_lac;
	      }
	      if (exists ($ppConfig->{COUNTGROUPFIELD}{$meas_group}{'T_CELL_RNC'}) &&  defined($t_cell_rnc)){
		my $field_pos=$ppConfig->{COUNTGROUPFIELD}{$meas_group}{'T_CELL_RNC'};
		$output->{$meas_time}{$perlen}{$meas_group}{$jobidx}{$field_pos} = $t_cell_rnc;
	      }

	      $servicemember_label="";
	      $linkset_label="";
	      $CGName = "";
	      $MediaGW_CS_Label ="";
            } else {
              messageHandler($config, 1, "Could not create full key for counter : ", $cnt_name
			     , " Measurement group ", $meas_group) 
            } # if defined $jobidx
	  } elsif ($create_jobidx_or_not->{$meas_group}{$moid} eq "done") {
            $jobidx = $create_jobidx_cache->{$meas_group}{$moid};
          } # if rewite jobidx
  
          if (defined $jobidx) {
            my $field_pos= $ppConfig->{COUNTGROUPFIELD}{$meas_group}{$cnt_name};
	    $output->{$meas_time}{$perlen}{$meas_group}{$jobidx}{$field_pos} = $cnt_value;
	  }
          
        } else {
          # Found counter in data which there is no mapping for
          # Only report an error if the counter is not in the COUNTS_TO_IGNORE file
          unless ((exists $ppConfig->{COUNTS_TO_IGNORE}{$moid_entity} and
		   exists $ppConfig->{COUNTS_TO_IGNORE}{$moid_entity}{$cnt_name}) or
		  exists $missing_counters{$moid_entity}{$cnt_name}) {
            messageHandler($config, 1, "No mapping found for counter : ",$cnt_name,", moid : ",$moid_entity);
	    $missing_counters{$moid_entity}{$cnt_name}= 1;
          }
        } # if COUNTER TO GROUP mapping 
      } # if $line_type eq "MOID"
    } # if $line_type eq "SN"
  } # while
} # sub build_data_hash

## --------------- End of build_data_hash sub routine--------

# initModules - provides any initialisation required for preLoadProcess modules
sub initModules {
  my ($rawDataFile, $config, $ppConfig) = @_ ;
  $ppConfig->{CONFIG} = $config ;
  $ppConfig->{ELEMENTLIST} = {} ;
  $ppConfig->{NODEID} = "unspecified" ;
  populate_subroutine_list($config);
  loadElementList($config,$ppConfig->{ELEMENTLIST});
  $ppConfig->{NODETYPE} = "CallServer"; # Hack!
  loadPPConfig($config, $ppConfig);
  load_counts_to_ignore($config, $ppConfig);
  $config->{PPCONFIG} = $ppConfig ;
} # sub initModules
## -------- End of initModules SubRoutine---------------------


# initialiseConfiguration - setup the configuration parameters for this module
sub initialiseConfiguration {

  my %user_defaults=
  (
    # Regex for pulling year, month, day, hours, minutes and seconds out of
    # a timestamp.
    INPUT_TSTAMP_FORMAT =>
    "(\\d{4})(\\d{2})(\\d{2})(\\d{2})(\\d{2})(\\d{2})",

    # Format to use with (s) printf to output timestamps in the format we want.
    OUTPUT_TSTAMP_FORMAT =>
    "%04d%02d%02d%02d%02d%02d",

    LOGFILENAME => "preProcessLog" . todays_date(),
    );

  register_config_defaults (%user_defaults);

  register_read_only_variables
    ("LOGFILENAME");

  register_mandatory_variables
    ("LOGFILEDIR",
     "INSEP",
     "OUTSEP",
     "CSFILENAMEPATTERN",
     "ELEMENTLIST",
     "LOADCONFIGFILE",
     "COUNTIGNOREFILE",
    );
}
## -------- End of initialiseConfiguration SubRoutine---------------------



sub loadElementList {
  my ($config, $elementList) = @_ ;
  my ($fileName)= $config->{ELEMENTLIST};
  my ($line) ;
  if (!(open(THEFILE, "$fileName"))) {
    fatalError ($config, "Could not open element list ($fileName) for reading: $!") ;
  }
  
  while (<THEFILE>) {
    chomp;
    next if /^\#/ or /^\s*$/; # Skip comments and blank lines.
    my ($element_name, $sys_id, $element_id, $release, $buff_size, $entity_type, $tech_type) = split ('\t', $_);

    if (not defined $tech_type) {
      messageHandler ($config, 1, "Element list ($fileName) corrupted for element : $element_name   Skipping ...");
      next;
    }

    if ($entity_type eq 'CallServer') {
      if ($element_id ne '') {
        $elementList->{$element_name}{ID} = $element_id ;
      } else {
        messageHandler($config, 1, "No element Id defined in element list ($fileName) for element: $element_name  Skipping ...");
      }
    }
  }
  close(THEFILE);
} # sub loadElementList

##------------End of loadElementList module------------------

sub loadPPConfig {
  my ($config, $ppConfig) = @_ ;
  my $fileName= $config->{LOADCONFIGFILE};

  $ppConfig->{GROUPFIELDNAME} = {} ;
  $ppConfig->{REMMAPPING} = {} ; # Required only for remediation
  if (!(open(THEFILE, "$fileName"))) {
    fatalError ($ppConfig, "Could not open ppConfig file ($fileName) for reading: $!") ;
  }

  my $line ;

  while ($line = <THEFILE>) {
    chomp $line ;
    next if ($line =~ /^\#/) ;
    next if ($line eq '') ;
    my ($counterNameData, $counterNameDD, $moid_entity, $measurementGroup,
	$groupField, $modList) = split(/\t/, $line);
    $modList =~ s/\"//g ; # Excel inserts quotes, so remove them

    if (length($counterNameData) > 0) {
      #SEAde68523 -make case not sensitives
      $counterNameData = uc $counterNameData;
      #print "counterNameData : $counterNameData\n";

      if (not defined $modList) {
        messageHandler($ppConfig, 1, "$fileName, line $.: Invalid line found in group mapping file - skipping");
        next;
      }

      if (defined $modList and $modList ne "" and $modList ne "none" and $modList ne "N/A" and
	  not exists ($config->{REWRITE_SUB_LIST}->{$modList})) {
	fatalError ($config, "$fileName, line $.: non-existent rewrite subroutine $modList used.");
      }

      if ($modList ne "N/A") {
	 if (exists $ppConfig->{MODLIST}{$measurementGroup} and
	     $modList ne $ppConfig->{MODLIST}{$measurementGroup}) {
	    messageHandler($ppConfig, 1, "$fileName, $.: Module list \"$modList\" for ",
			   "counter $counterNameDD is different ",
			   "from previous values for measurement group ",
			   "$measurementGroup - ignoring.");
         } else {
	    $ppConfig->{MODLIST}{$measurementGroup}= $modList;
         }
      }

      if (!exists($ppConfig->{GROUPFIELDNAME}{$measurementGroup}{maxField}) ||
         ($groupField > $ppConfig->{GROUPFIELDNAME}{$measurementGroup}{maxField})) {
        $ppConfig->{GROUPFIELDNAME}{$measurementGroup}{maxField} = $groupField ;
      }

      if (exists ($ppConfig->{COUNTGROUPFIELD}{$measurementGroup}{$counterNameData})) {
        messageHandler ($ppConfig, 1, "$fileName, $.: Group field value $groupField for ",
                        "counter $counterNameDD duplicates a previous ",
                        "counter name in measurement group $measurementGroup.");
        # We still store it, because this is a way to handle counters
        # being renamed.
      }
      $ppConfig->{COUNTGROUPFIELD}{$measurementGroup}{$counterNameData} = $groupField;

      $ppConfig->{GROUPFIELDNAME}{$measurementGroup}{field}{$groupField}= $counterNameDD;
      for my $entity (split '\|',$moid_entity) {
        $ppConfig->{COUNTER_TO_GROUP}{$entity}{$counterNameData} = $measurementGroup;
      }

      # This allows for the grouping of counters in the data to be different
      # from the grouping of counters in Prospect.
      $ppConfig->{SHORT_COUNTER_TO_GROUP}{$counterNameDD}= $measurementGroup;
    } # if length $counterNameData > 0
  } # while $line = <THEFILE>
  close(THEFILE);

} # sub loadPPConfig

sub load_counts_to_ignore {
  my ($config, $ppConfig) = @_ ;
  my $fileName=$config->{COUNTIGNOREFILE};

  if (!(open(THEFILE, "$fileName"))) {
    fatalError ($ppConfig, "Could not open counts to ignore file ($fileName) for reading: $!") ;
  }

  my $line ;
  while ($line = <THEFILE>) {
    chomp $line ;
    next if ($line =~ /^\#/) ;
    next if ($line eq '') ;
    my ($counterNameData, $moid_entity) = split(/\t/, $line);

    $ppConfig->{COUNTS_TO_IGNORE}{$moid_entity}{$counterNameData} = "ignore";
  }
} # sub load_counts_to_ignore 

## ------------End of loadPPConfig module-------------------

sub showGeneral {
  my ($fileName, $measTime, $perlen, $ppConfig, $config, $outsep) = @_ ;
  $measTime= round_tstamp_to_period($config, $measTime, $perlen);
  print join ($outsep,
	      "MEASGROUP_CallServer_General",
	      "5",
	      "FILENAME",
	      "TIMESTAMP",
	      "PERLENSEC",
              "LABEL",
              "INSTANCE_NAME",
	      "\n"
	     );
  print join ($outsep,
	      "CallServer",
	      "",
	      $fileName,
	      $measTime,
	      $perlen,
              $ppConfig->{CallServer_LABEL},
              $ppConfig->{CallServer_NAME},
	      "\n"
	     );
} # sub showGeneral

## ------------End of showGeneral SubRoutine---------------


## Subroutine to parse the file name extract the timestamp and CallServer ID
sub parse_filename {
  my ($file_name, $config, $ppConfig) = @_;

  my ($CallServer_name, $CallServer_id, $meas_time);

  ($CallServer_name, $meas_time) = $file_name =~ /$config->{CSFILENAMEPATTERN}/o;

  if (not defined $meas_time) {
    messageHandler ($config, 1, "Could not extract Call Server name and meas time from filename.  Filename : ",$file_name);
    return (undef, undef);
  }

  # Add seconds to the measurement time
  $meas_time = $meas_time . "00";

  if (exists $ppConfig->{ELEMENTLIST}{$CallServer_name}) {
    $CallServer_id = $ppConfig->{ELEMENTLIST}{$CallServer_name}{ID};
  } else {
    messageHandler ($config, 1, "Could not find CallServer name in element list : ",$CallServer_name);
    return (undef, undef);
  }

  return ($CallServer_name, $CallServer_id, $meas_time);
}
##--------End of parse_filename-----------------

sub create_idx_ServiceMember {
  my ($config, $CallServer_id, $moid, $original_moid) = @_;

  unless (defined $original_moid) {
    $original_moid = $moid;
  }
    
  $moid =~ s/,TechnologyType=.+$//;

  my ($cabinet, $shelf, $card, $host);

  if ($moid =~ /(.+),serviceType/){
   (my $current_moid, undef) = $moid =~ /(.+),serviceType=(.+)$/;
   ($cabinet, $shelf, $card, $host, undef) = $moid =~ /Cabinet=(.+),Shelf=(.+),Card=(.+),Host=(.+),serviceType=(.+)$/;
  }
  elsif ($moid =~ /Cabinet=(.+),Shelf=(.+),Card=(.+),Host=(.+)$/){
    ($cabinet, $shelf, $card, $host) = ($1, $2, $3, $4); 
  }
  else{
   ($cabinet, $shelf, $card, $host) = $moid =~ /Cabinet_(.+)_Shelf_(.+)_Card_(.+)_Host_(.+)$/;
  }
    

  if (not defined $host) {
    messageHandler ($config, 1, "Could not extract cabinet,shelf,card and host from moid : $original_moid");
    return undef;
  }

  return "${CallServer_id}-${cabinet}:${shelf}:${card}:${host}";
}

sub create_idx_SNMP_Traps {
  my ($config, $CallServer_id, $moid, $original_moid) = @_;

  unless (defined $original_moid) {
    $original_moid = $moid;
  }
    
  my ($ip) = $moid =~ /^IPdest=(.*)$/;

  if (not defined $ip) {
    messageHandler ($config, 1, "Could not extract IP address from moid : $original_moid");
    return undef;
  }

  return "${CallServer_id}-${ip}";
}

sub create_idx_SubNE {
  my ($config, $CallServer_id, $moid, $original_moid) = @_;
  return create_idx($config, $CallServer_id, $moid,
			  "IPsne", "IPsne", "SNMP_Traps", $original_moid);
}

sub create_idx_BearerType {
  my ($config, $CallServer_id, $moid, $original_moid) = @_;
  return create_idx($config, $CallServer_id, $moid,
			  "BearerCallType", "bearer call type", "ServiceMember", $original_moid);
}

sub create_idx_CPU_Core {
  my ($config, $CallServer_id, $moid, $original_moid) = @_;
  return create_idx($config, $CallServer_id, $moid,
			  "Core", "CPU_Core", "ServiceMember", $original_moid);
}

sub create_idx_ECType {
  my ($config, $CallServer_id, $moid, $original_moid) = @_;
  return create_idx($config, $CallServer_id, $moid,
			  "ECT", "ECType", "ServiceMember", $original_moid);
}

sub create_idx_HO_Cause {
  my ($config, $CallServer_id, $moid, $original_moid) = @_;
  return create_idx($config, $CallServer_id, $moid,
		    "CauseValue", "cause", "ServiceMember", $original_moid);
}

sub create_idx_HONPool {
  my ($config, $CallServer_id, $moid, $original_moid) = @_;
  return create_idx($config, $CallServer_id, $moid,
			  "HONPool", "HONPool", "ServiceMember", $original_moid);
}

sub create_idx_MSRNPool {
  my ($config, $CallServer_id, $moid, $original_moid) = @_;
  return create_idx($config, $CallServer_id, $moid,
			  "MSRNPool", "MSRNPool", "ServiceMember", $original_moid);
}

sub create_idx_MTP_Stack {
  my ($config, $CallServer_id, $moid, $original_moid) = @_;
  return create_idx($config, $CallServer_id, $moid,
			  "Stack", "MTP_Stack", "ServiceMember", $original_moid);
}

sub create_idx_SCCP_DPC {
  my ($config, $CallServer_id, $moid, $original_moid) = @_;
  return create_idx($config, $CallServer_id, $moid,
			  "DPC", "SCCP_DPC", "MTP_Stack", $original_moid);
}

sub create_idx_ChannelGroup {
  my ($config, $CallServer_id, $moid, $original_moid) = @_;
  return create_idx($config, $CallServer_id, $moid,
		    "ChannelGroup", "channel group", "ServiceMember", $original_moid);
}

sub create_idx_IN_Service {
  my ($config, $CallServer_id, $moid, $original_moid) = @_;
  return create_idx($config, $CallServer_id, $moid,
		    "IN", "in service", "ServiceMember", $original_moid);
}

sub create_idx_SigPoint_DestGT {
  my ($config, $CallServer_id, $moid, $original_moid) = @_;
  return create_idx($config, $CallServer_id, $moid,
		    "CDPA", "SigPoint_DestGT", "ServiceMember", $original_moid);
}

sub create_idx_SigPoint_DestPC {
  my ($config, $CallServer_id, $moid, $original_moid) = @_;
  return create_idx($config, $CallServer_id, $moid,
  #RP8
  #SEAde68512
  #support new measObjLdn
  #Service=ss7, ServiceMember=Cabinet_<p>_Shelf_<q>_Card_<r>_Host_<s>, SSN=<x>
  #"PointCode|DPC", "SigPoint_DestPC", "ServiceMember", $original_moid);
    "PointCode|DPC|SSN", "SigPoint_DestPC", "ServiceMember", $original_moid);
}

sub create_idx_SigPoint_DestPC_CIC {
  my ($config, $CallServer_id, $moid, $original_moid) = @_;
  return create_idx($config, $CallServer_id, $moid,
		    "CIC", "SigPoint_DestPC_CIC", "SigPoint_DestPC", $original_moid);
}

sub create_idx_SigPoint_DestPC_SSN {
  my ($config, $CallServer_id, $moid, $original_moid) = @_;
  return create_idx($config, $CallServer_id, $moid,
		    "SSN", "SigPoint_DestPC_SSN", "SigPoint_DestPC", $original_moid);
}

sub create_idx_SigPoint_SCCP {
  my ($config, $CallServer_id, $moid, $original_moid) = @_;
  return create_idx($config, $CallServer_id, $moid,
		    "SCCP", "SigPoint_SCCP", "ServiceMember", $original_moid);
}

sub create_idx_SigPoint_SCCP_SSN {
  my ($config, $CallServer_id, $moid, $original_moid) = @_;
  return create_idx($config, $CallServer_id, $moid,
		    "SSN", "SigPoint_SCCP_SSN", "SigPoint_SCCP", $original_moid);
}

sub create_idx_SigPoint_TCAP {
  my ($config, $CallServer_id, $moid, $original_moid) = @_;
  return create_idx($config, $CallServer_id, $moid,
		    "TCAP", "SigPoint_TCAP", "ServiceMember", $original_moid);
}

sub create_idx_TCAP_SSN {
  my ($config, $CallServer_id, $moid, $original_moid) = @_;
  return create_idx($config, $CallServer_id, $moid,
		    "SSN", "TCAP", "SigPoint_TCAP", $original_moid);
}

sub create_idx_VM {
  my ($config, $CallServer_id, $moid, $original_moid) = @_;
  return create_idx($config, $CallServer_id, $moid,
		    "VM", "Virtual Machine", "ServiceMember", $original_moid);
}

sub create_idx_SigTran_SG {
  my ($config, $CallServer_id, $moid, $original_moid) = @_;
  return create_idx($config, $CallServer_id, $moid,
		    "M3UASG", "SigTran_SG", "ServiceMember", $original_moid);
}

sub create_idx_SigTran_SCTPAssoc {
  my ($config, $CallServer_id, $moid, $original_moid) = @_;
  return create_idx($config, $CallServer_id, $moid,
		    "SCTPAssoc", "SigTran_SCTPAssoc", "ServiceMember", $original_moid);
}

sub create_idx_SNMP_Interface{
  my ($config, $CallServer_id, $moid, $original_moid) = @_;

  unless (defined $original_moid) {
    $original_moid = $moid;
  }
    
  my ($cabinet, $shelf, $slot, $card, $host, $oid, $counter);

  ($cabinet, $shelf, $slot, $card, $host, $oid, $counter) = $moid =~ /CesCabinet=(.+),CesShelf=(.+),CesSlot=(.+),CesCard=(.+),CesHost=(.+),oid=(.*?).(\w+)$/;

  if (not defined $host) {
    messageHandler ($config, 1, "Could not extract cescabinet,cesshelf,cescard and ceshost from moid : $original_moid");
    return undef;
  }

  return "${CallServer_id}-${cabinet}:${shelf}:${slot}:${card}:${host}-${counter}";
}

sub create_idx_SCTP_Stack {
  my ($config, $CallServer_id, $moid, $original_moid) = @_;

  $original_moid = $moid unless defined $original_moid;
  my ($cesservicemember, $counter) =
    $moid =~ /CesService=.+,CesServiceMember=(.+),oid=\.iso\.org\.dod\.internet\.mgmt\.mib-(.+)$/;

  if (not defined $cesservicemember) {
    messageHandler ($config, 1, "Could not extract CesServiceMember and counter name from moid : $original_moid");
    return undef;
  }
  $cesservicemember =~ tr/-/_/;
  return "${CallServer_id}-${cesservicemember}-${counter}";
}

sub create_idx_PDF{
  my ($config, $CallServer_id, $moid, $original_moid) = @_;
  return create_idx($config, $CallServer_id, $moid,
		    "PDF", "PDF", "ServiceMember", $original_moid);
}

sub create_idx_HSL {
  my ($config, $CallServer_id, $moid, $original_moid) = @_;
  return create_idx($config, $CallServer_id, $moid,
		    "HSL", "HSL", "LinkSet", $original_moid);
}

sub create_idx_Disk {
  my ($config, $CallServer_id, $moid, $original_moid) = @_;
  return create_idx($config, $CallServer_id, $moid,
		    "Disk", "disk", "ServiceMember", $original_moid);
}

sub create_idx_FileSystem {
  my ($config, $CallServer_id, $moid, $original_moid) = @_;
  return create_idx($config, $CallServer_id, $moid,
		    "FileSystem", "file system", "ServiceMember", $original_moid);
}

sub create_idx_LAC_Paging {
  my ($config, $CallServer_id, $moid, $original_moid) = @_;
  return create_idx($config, $CallServer_id, $moid,
		    "LAC", "LAC", "ServiceMember", $original_moid);
}

sub create_idx_LinkSet {
  my ($config, $CallServer_id, $moid, $original_moid) = @_;
  if ($moid =~ /^(.+),LSName=.+$/){
    my ($current_moid) = ($1);
    $moid = $current_moid;
  }
  return create_idx($config, $CallServer_id, $moid,
		    "Linkset", "link set", "ServiceMember", $original_moid);
}

sub create_idx_MediaResourceServer {
  my ($config, $CallServer_id, $moid, $original_moid) = @_;
  return create_idx($config, $CallServer_id, $moid,
		    "MSID|MediaResourceServer", "media resource server", "ServiceMember", $original_moid);
}

sub create_idx_MGW_CS {
  my ($config, $CallServer_id, $moid, $original_moid) = @_;
  return create_idx($config, $CallServer_id, $moid,
		    "Mgw", "media gateway", "ServiceMember", $original_moid);
}

sub create_idx_MGW_BearerType_CS {
  my ($config, $CallServer_id, $moid, $original_moid) = @_;
  return create_idx($config, $CallServer_id, $moid,
		    "BearerType", "bearer type", "MGW_CS", $original_moid);
}

sub create_idx_MGW_CmdType_CS {
  my ($config, $CallServer_id, $moid, $original_moid) = @_;
  return create_idx($config, $CallServer_id, $moid,
		    "CmdType", "command type", "MGW_CS", $original_moid);
}

sub create_idx_MGW_ErrorCode_CS {
  my ($config, $CallServer_id, $moid, $original_moid) = @_;
  return create_idx($config, $CallServer_id, $moid,
		    "ErrorCode", "error code", "MGW_CS", $original_moid);
}

sub create_idx_ChannelGroup_MGW {
  my ($config, $CallServer_id, $moid, $original_moid) = @_;
  return create_idx($config, $CallServer_id, $moid,
		    "Mgw", "MGW", "ChannelGroup", $original_moid);
}

sub create_idx_Link {
  my ($config, $CallServer_id, $moid, $original_moid) = @_;
  return create_idx($config, $CallServer_id, $moid,
		    "Link", "link", "LinkSet", $original_moid);
}

sub create_idx_PLMN {
  my ($config, $CallServer_id, $moid, $original_moid) = @_;
  return create_idx($config, $CallServer_id, $moid,
		    "PLMN", "PLMN", "ServiceMember", $original_moid);
}

sub create_idx_LAC {
  my ($config, $CallServer_id, $moid, $original_moid) = @_;
  return create_idx($config, $CallServer_id, $moid,
		    "LAC", "LAC", "PLMN", $original_moid);
}

sub create_idx_GSM_Cell {
  my ($config, $CallServer_id, $moid, $original_moid) = @_;
  return create_idx($config, $CallServer_id, $moid,
		    "CELL|CellID", "cell", "LAC", $original_moid);
}

sub create_idx_RNC_CS {
  my ($config, $CallServer_id, $moid, $original_moid) = @_;
  return create_idx($config, $CallServer_id, $moid,
		    "RNC|RNCID", "RNC", "LAC", $original_moid);
}

#
# add for LucentUMTS_RP8P20
sub create_idx_RNC_BearerType {
  my ($config,  $CallServer_id, $moid, $original_moid) = @_;
  return create_idx($config, $CallServer_id, $moid,
		    "BearerCallType", "bearer call type", "ServiceMember", $original_moid);
}

sub create_idx_BSC {
  my ($config,  $CallServer_id, $moid, $original_moid) = @_;
  return create_idx($config, $CallServer_id, $moid,
		    "BSC", "BSC", "ServiceMember", $original_moid);
}

sub create_idx_BSC_SM_CS {
  my ($config,  $CallServer_id, $moid, $original_moid) = @_;
  return create_idx($config, $CallServer_id, $moid,
		    "BSC", "BSC", "ServiceMember", $original_moid);
}

sub create_idx_BSC_BearerType_CS {
  my ($config,  $CallServer_id, $moid, $original_moid) = @_;
  return create_idx($config, $CallServer_id, $moid,
		    "BearerCallType", "bearer call type", "BSC", $original_moid);
}

sub create_idx_LAC_BSC_BearerType {
  # Can't use the generic method of calling create_idx
  # with the appropriate parent entity, because when that sees
  # "LAC", it will assume the MOID should look like
  # ServiceMember-PLMN-LAC,
  # which this doesn't.
  my ($config,  $CallServer_id, $moid, $original_moid) = @_;
  $original_moid= $moid unless defined $original_moid;
  my ($start_moid, $lac_paging, $bsc, $bearertype)= 
    $moid =~ /(.+),LAC=(.+),BSC=(.+),BearerCallType=(.+)$/;
  if (not defined $start_moid) {
    messageHandler($config, 1, "Could not extract LAC, BSC and BearerCallType from moid: $original_moid");
    return undef;
  }
  my ($servicemember_moid)= 
    create_idx_ServiceMember($config, $CallServer_id, $start_moid, $start_moid);
  if (defined $servicemember_moid) {
    return $servicemember_moid . "-$lac_paging-$bsc-$bearertype";
  } else {
    return undef;
  }
} # create_idx_LAC_BSC_BearerType

sub create_idx_LAC_RNC_BearerType {
  # Can't use the generic method of calling create_idx
  # with the appropriate parent entity, because when that sees
  # "LAC", it will assume the MOID should look like
  # ServiceMember-PLMN-LAC,
  # which this doesn't.
  my ($config,  $CallServer_id, $moid, $original_moid) = @_;
  $original_moid= $moid unless defined $original_moid;
  my ($start_moid, $lac_paging, $rnc, $bearertype)= 
    $moid =~ /(.+),LAC=(.+),RNC=(.+),BearerCallType=(.+)$/;
  if (not defined $start_moid) {
    messageHandler($config, 1, "Could not extract LAC, RNC and BearerCallType from moid: $original_moid");
    return undef;
  }
  my ($servicemember_moid)= 
    create_idx_ServiceMember($config, $CallServer_id, $start_moid, $start_moid);
  if (defined $servicemember_moid) {
    return $servicemember_moid . "-$lac_paging-$rnc-$bearertype";
  } else {
    return undef;
  }
} # create_idx_LAC_RNC_BearerType

sub create_idx_LAC_PG_BearerType {
  # Can't use the generic method of calling create_idx
  # with the appropriate parent entity, because when that sees
  # "LAC", it will assume the MOID should look like
  # ServiceMember-PLMN-LAC,
  # which this doesn't.
  my ($config,  $CallServer_id, $moid, $original_moid) = @_;
  $original_moid= $moid unless defined $original_moid;
  my ($start_moid, $lac, $bearertype)= 
    $moid =~ /(.+),LAC=(.+),BearerCallType=(.+)$/;
  if (not defined $start_moid) {
    messageHandler($config, 1, "Could not extract LAC and BearerCallType from moid: $original_moid");
    return undef;
  }
  my ($servicemember_moid)= 
    create_idx_ServiceMember($config, $CallServer_id, $start_moid, $start_moid);
  if (defined $servicemember_moid) {
    return $servicemember_moid . "-$lac-$bearertype";
  } else {
    return undef;
  }
} # create_idx_LAC_PG_BearerType

sub create_idx_RNC_BearerType_CS {
  my ($config,  $CallServer_id, $moid, $original_moid) = @_;
  $original_moid= $moid unless defined $original_moid;
  my ($start_moid, $rnc, $bearertype)= 
    $moid =~ /(.+),RNC=(.+),BearerCallType=(.+)$/;
  if (not defined $start_moid) {
    messageHandler($config, 1, "Could not extract RNC and BearerCallType from moid: $original_moid");
    return undef;
  }
  my ($servicemember_moid)= 
    create_idx_ServiceMember($config, $CallServer_id, $start_moid, $start_moid);
  if (defined $servicemember_moid) {
    return $servicemember_moid . "-$rnc-$bearertype";
  } else {
    return undef;
  }
} # create_idx_RNC_BearerType_CS

sub create_idx_LAC_BSC {
  my ($config,  $CallServer_id, $moid, $original_moid) = @_;
  return create_idx($config, $CallServer_id, $moid,
		    "BSC", "BSC", "LAC", $original_moid);
}

sub create_idx_GSM_Target {
  my ($config,  $CallServer_id, $moid, $original_moid) = @_;
  return create_idx($config, $CallServer_id, $moid,
		    "Target", "target", "GSM_Cell", $original_moid);
}

sub create_idx_UMTS_Target {
  my ($config,  $CallServer_id, $moid, $original_moid) = @_;
  return create_idx($config, $CallServer_id, $moid,
		    "Target", "target", "RNC_CS", $original_moid);
}

sub create_idx_UMRF{
  my ($config, $CallServer_id, $moid, $original_moid ) = @_;
  return create_idx($config, $CallServer_id, $moid,
		    "UMRF", "umrf", "ServiceMember", $original_moid);
}

sub create_idx_mediagw_cs{
  my ($config, $CallServer_id, $moid, $original_moid) = @_;

  my ($mgwid, $mgwname) = $moid =~ /^MgwID=(.+),mgwName=(.+)$/;
  return "${CallServer_id}-${mgwid}";
}

sub create_idx_s_cell_rnc_sac{
  my ($config, $CallServer_id, $moid, $original_moid) = @_;
  return create_idx($config, $CallServer_id, $moid,
		    "S_RNCID|S_SAC|S_CellID|S_RNC|S_Cell|S_CELL", "s_cell_rnc_sac", "ServiceMember", $original_moid);
}

sub create_idx_t_cell_rnc{
  my ($config, $CallServer_id, $moid, $original_moid) = @_;
  return create_idx($config, $CallServer_id, $moid,
		    "T_CellID|T_RNCID|T_CELL|T_Cell|T_RNC", "t_cell_rnc", "s_cell_rnc_sac", $original_moid);
}

sub create_idx_t_c_r_cause{
  my ($config, $CallServer_id, $moid, $original_moid) = @_;
  return create_idx($config, $CallServer_id, $moid,
		   "U_Cause|G_Cause|MAP_Cause", "t_c_r_cause", "t_cell_rnc", $original_moid);
}

sub create_idx_rnc_sm_cs{
  my ($config, $CallServer_id, $moid, $original_moid) = @_;
  return create_idx($config, $CallServer_id, $moid,
		    "RNC", "rnc_sm_cs", "ServiceMember", $original_moid);
}

sub create_idx_mgw_codectype_cs{
  my ($config, $CallServer_id, $moid, $original_moid) = @_;
  
  return create_idx($config, $CallServer_id, $moid,
		   "CodecType", "mgw_codectype_cs", "MGW_CS", $original_moid);
}

sub create_idx_cg_mgw_bearertype{
  my ($config, $CallServer_id, $moid, $original_moid) = @_;
  my ($current_moid, $rest_moid) = $moid =~ /(.+),CGName=.+,(.+)/;
  $moid = $current_moid . "," . $rest_moid;
  return create_idx($config, $CallServer_id, $moid,
                   "BearerType", "cg_mgw_bearertype", "ChannelGroup_MGW", $original_moid);
}

sub create_idx_iwf_gw_cs{
  my ($config, $CallServer_id, $moid, $original_moid) = @_;
  return create_idx($config, $CallServer_id, $moid,
		    "IWF Gateway ID", "iwf_gw_cs", "ServiceMember", $original_moid);
}
sub create_idx_sac_cs{
  my ($config, $CallServer_id, $moid, $original_moid) = @_;
  return create_idx($config, $CallServer_id, $moid,
		    "SAC", "sac_cs", "LAC", $original_moid);
}

### Subroutine to create full key for child entities of ServiceMember
sub create_idx {
  my ($config, $CallServer_id, $moid, $search_moid, $message, $parent_entity, $original_moid) = @_;

  unless (defined $original_moid) {
    $original_moid = $moid;
  }

  my $rest_moid="";
  my $local_moid="";
  #messageHandler ($config, 3, "moid:[$moid], search_moid:[$search_moid], parent_entity:[$parent_entity]"); 
  if ($moid =~ /(.+),U_Cause=(.+)$/){
    my ($other_moid, $cause) = ($1, $2);
    $local_moid = sprintf("UMTS:%s",$cause);
    $rest_moid = $other_moid;
  }
  elsif ($moid =~ /(.+),G_Cause=(.+)$/){
    my ($other_moid, $cause) = ($1, $2);
    $local_moid = sprintf("GSM:%s",$cause);
    $rest_moid = $other_moid;
  }
  elsif ($moid =~ /(.+),MAP_Cause=(.+)$/){
    my ($other_moid, $cause) = ($1, $2);
    $local_moid = sprintf("MAP:%s",$cause);
    $rest_moid = $other_moid;
  }
  elsif ($moid =~ /(.+),T_PLMN=(.+),[^=]+=(.+),[^=]+=(.+)$/){
    my ($other_moid, $plmn, $lacid, $tag, $rncid) = $moid =~ /(.+),T_PLMN=(.+),[^=]+=(.+),(.+)=(.+)$/; 
    if ( $tag =~ /(T_CellID|T_CELL|T_Cell)/){
      $local_moid = sprintf("%s:%s:CELL:%s",$plmn,$lacid,$rncid);
    } 
    elsif ( $tag =~ /(T_RNCID|T_RNC)/){
      $local_moid = sprintf("%s:%s:RNC:%s",$plmn,$lacid,$rncid);
    }
    $rest_moid = $other_moid;
  }
  elsif ($moid =~ /(.+),S_PLMN=(.+),[^=]+=(.+),[^=]+=(.+)$/){
    my ($other_moid, $plmn, $lacid, $tag, $rncid) = $moid =~ /(.+),S_PLMN=(.+),[^=]+=(.+),(.+)=(.+)$/; 
    if ( $tag =~ /(S_RNCID|S_RNC)/){
      $local_moid = sprintf("%s-%s-RNC:%s",$plmn,$lacid,$rncid);
    }
    elsif ( $tag =~ /(S_CellID|S_CELL|S_Cell)/){
      $local_moid = sprintf("%s-%s-CELL:%s",$plmn,$lacid,$rncid);
    }
    if ( $tag =~ /(S_SACID|S_SAC)/){
      $local_moid = sprintf("%s-%s-SAC:%s",$plmn,$lacid,$rncid);
    }
    $rest_moid = $other_moid;
  }
  #
  # add for LucentUMTS_RP8P20
  # take care <PLMN>-<LAC>-<RNC>-<BearerCallType>
  #
  elsif ($moid =~ /(.+),PLMN=(.+),LAC=(.+),RNC=(.+),BearerCallType=(.+)$/){
    my ($other_moid, $plmn, $lac, $rnc, $btype) = ($1, $2, $3, $4, $5);
    $local_moid = sprintf("%s-%s-%s-%s",$plmn,$lac,$rnc,$btype);
    $rest_moid = $other_moid;
  }
  else{
    # The brackets around $search_moid allow us to match multiple words e.g. CELL|CellID
    my ($other_moid, $search, $tmp_moid) = $moid =~ /^(.+),($search_moid)=(.+)$/;
    $local_moid=$tmp_moid;
    $rest_moid = $other_moid;

    #LuUMTS RP4
    #specific to measObjLdn Tag like Service=....., Mgw=<y>, ChannelGroup=<x>
    #modify if to Service=....., ChannelGroup=<x>, Mgw=<y>
    if ($moid =~ /^(.+),Mgw=(.+),ChannelGroup=(.+),(.+)$/){
      ##purposely put this if condition although for doing nothing
      ##to avoid moid cg_mgw_bearertype go to moid channelgroup_mgw
    }
    elsif ($moid =~ /^(.+),Mgw=(.+),ChannelGroup=(.+)$/){
      ($local_moid, my $t_moid) = $local_moid =~ /^(.+),(.+)$/;
      $rest_moid = $rest_moid . "," . $t_moid; 
    }
    #LuUMTS RP6
    #specific to measObjLdn Tag like Service=....., CIC=<y>, DPC=<x>
    #modify if to Service=....., DPC=<x>, CIC=<y>
    if ($moid =~ /^(.+),CIC=(.+),DPC=(.+)$/){
      ($local_moid, my $t_moid) = $local_moid =~ /^(.+),(.+)$/;
      $rest_moid = $rest_moid . "," . $t_moid; 
    }
  }


  #end LuUMTS RP4
  if (not defined $local_moid) {
    messageHandler ($config, 1, "Could not extract $message Id from moid : $original_moid");
    return undef;
  }

  $local_moid =~ s/^ +//;
  my $parent_moid;
  my $modList= "create_idx_" . $parent_entity;
  if (exists $config->{REWRITE_SUB_REFS}->{$modList}) {
    $parent_moid= &{$config->{REWRITE_SUB_REFS}->{$modList}}
    ($config, $CallServer_id, $rest_moid, $moid);
  } else {
    fatalError ($config, "Unknown parent entity : $parent_entity");
    # Perl would abort the script anyway if we tried to call a 
    # non-existent subroutine, but doing it this way gives a more meaningful
    # error message.
  }

  if (not defined $parent_moid) {
    return undef;
  }

  return "${parent_moid}-${local_moid}";
}

sub populate_subroutine_list {
  # Obtain a list of all the subroutines within the program.

  my ($config)= @_;
  foreach my $i (keys %main::) {
    # The loop retrieves all global identifiers within the module,
    # so we need to filter it to hold only subroutines.
    local *sym= $main::{$i};
    if (defined &sym) {
      $config->{REWRITE_SUB_LIST}->{$i}= 1;
      # Take a reference to this subroutine (like a function pointer in C),
      # so that other subroutines can call it
      $config->{REWRITE_SUB_REFS}->{$i}= \&{$main::{$i}};
    }
  } # foreach $i
} # sub populate_subroutine_list

##Return true to show that the module loaded successfully.
1;
